/* Copyright 2000 Red Hat, Inc.
 *
 * Than Ngo <than@redhat.de>
 *
 * This software may be freely redistributed under the terms of the GNU
 * public license.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


#ifndef QTONLY
#  include <klocale.h>
#  include <kiconloader.h>
#else
#  include "kde-int.h"
#endif

#include <qlayout.h>
#include <qpushbutton.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qtooltip.h>
#include <qmessagebox.h>

#include <stdlib.h>
#include <iostream>

#include "dialout.h"
#include "generaldata.h"

#define ISDNINFO "/dev/isdninfo"

Dialout::Dialout(QWidget * parent, const char *name):QWidget(parent, name)
{
  currrentisdndevice = "";
  currrentphonenumber = "0";
  currrentprovidername = "";
  onlinecount = 0;
  is_isdnstatustimer = false;
  color = green;
  isdnstatestr = "  OFFLINE  ";

  connecttimer = new QTimer(this);
  connect(connecttimer, SIGNAL(timeout()), this, SLOT(slotcheckconnect()));
  connecttimer->start(200, false);

  isdnstatustimer = new QTimer(this);
  connect(isdnstatustimer, SIGNAL(timeout()), this, SLOT(slotcheckonlinestatus()));

  onlinetime = new QTime(0, 0, 0);
  
  filldialog();
  slotcheckconnect();
}

void Dialout::removeprovidername(const QString &s)
{
  uint index;
  for (index = 0; index < provider->count(); index++)
    if (provider->text(index) == s)
      break;

  provider->removeItem(index);
  slotcheckconnect();
}

void Dialout::addprovidername(const QString &s)
{
  provider->insertItem(KGlobal::iconLoader()->loadIcon("isp"), s);
  slotcheckconnect();
}

void Dialout::changeprovidername(const QString &_old, const QString &_new)
{
  for (uint i = 0; i < provider->count(); i++)
    if (provider->text(i) == _old)
      provider->changeItem(KGlobal::iconLoader()->loadIcon("isp"), _new, i);
  slotcheckconnect();
}

void Dialout::slotconnect()
{
  QString command;

  currrentisdndevice = "ippp" + QString::number(provider->currentItem());
  currrentphonenumber = ISDNData.getphonenumber(provider->currentItem());
  setisdnstatus(DIAL);
  command = "/usr/sbin/isdnctrl dial " + currrentisdndevice + " >& /dev/null";

  if (system(command) == 0) {
    b_connect->setEnabled(false);
    emit setproviderisdncardtab(false);
  }
  else {
    setisdnstatus(OFFLINE);
    errormessage();
  }
}

void Dialout::slothangup()
{
  QString command;
  command = "/usr/sbin/isdnctrl hangup " + currrentisdndevice + " >& /dev/null";
  if (system(command) == 0)
    {
      b_hangup->setEnabled(false);
      isdnstatustimer->stop();
      is_isdnstatustimer = false;
      emit setproviderisdncardtab(true);
    }
}

void Dialout::isdnscriptfailmessage()
{
  QString s = i18n("/etc/init.d/isdn is not available.\n"
                   "ISDN service cannot be started or stopped.\n\n"
                   "Please upgrade the isdn-config package and\n"
                   "try to configure your ISDN service again.");
  QMessageBox::warning(this, i18n("Fatal Error"), s);
}

void Dialout::loadhisaxmodulfailmessage()
{
  QString s = i18n("Loading the HiSax module failed.\n");
  QMessageBox::warning(this, i18n("Fatal Error"), s);
}

void Dialout::slotisdnstart()
{
  if (QFile("/etc/init.d/isdn").exists())
    {
      system("/etc/init.d/isdn start >& /dev/null");
      readisdninfo();
      if (!isdninfoidmap.contains("HiSax"))
        loadhisaxmodulfailmessage();
    }
  else
    isdnscriptfailmessage();

  slotcheckconnect();
}

void Dialout::slotisdnstop()
{
  if (QFile("/etc/init.d/isdn").exists())
    system("/etc/init.d/isdn stop >& /dev/null");
  else
    isdnscriptfailmessage();

  slotcheckconnect();
}

void Dialout::slotcheckconnect()
{
  readisdninfo();
  bool providerisdncardtabstatus = true;

  // no hisax is loaded
  if (!isdninfoidmap.contains("HiSax")) {
    b_connect->setEnabled(false);
    b_hangup->setEnabled(false);
    b_isdnstart->setEnabled(true);
    b_isdnstop->setEnabled(false);
    setisdnstatus(OFFLINE);
    isdnstatustimer->stop();
  }
  else { // hisax was loaded
    b_isdnstop->setEnabled(true);
    b_isdnstart->setEnabled(false);
    if (!isdninfophone.contains(currrentphonenumber)) {
      b_hangup->setEnabled(false);
      setisdnstatus(OFFLINE);
      isdnstatustimer->stop();
      if (provider->currentItem() < 0)
        b_connect->setEnabled(false); // no provider selected
      else
        b_connect->setEnabled(true); // provider selected
    }
    else { // online or dialing
      if (!is_trying()) // online
        setisdnstatus(ONLINE);
      else // dialing
        setisdnstatus(DIAL);

      // the line is on, disable provider and isdncard tabwidget
      b_connect->setEnabled(false);
      b_hangup->setEnabled(true);
      providerisdncardtabstatus = false;
    }
  }
  emit setproviderisdncardtab(providerisdncardtabstatus);
}

// set ISDN state
void Dialout::setisdnstatus(uint state)
{
  switch (state)
    {
    case ONLINE:
      isdnstatuslabel->setEraseColor(QColor("Red"));
      isdnstatestr = i18n("  ONLINE  ");
      break;
    case DIAL:
      isdnstatuslabel->setEraseColor(QColor("Yellow"));
      isdnstatestr = i18n("  DIALING  ");
      break;
//    case OFFLINE:
    default:
      isdnstatuslabel->setEraseColor(QColor("Green"));
      isdnstatestr = i18n("  OFFLINE  ");
      break;
    }
     
  isdnstatuslabel->setText(isdnstatestr);
}

void Dialout::slotcheckonlinestatus()
{
  onlinetime->addSecs(onlinecount++);
  QTime t = onlinetime->addSecs(onlinecount);
  timeconnectedlabel->setText("  " + t.toString() + "  ");
}

// reset the online counter
void Dialout::slotresetcounter()
{
  onlinecount = 0;
  timeconnectedlabel->setText("  00:00:00  ");
}

// read isdninfo from /dev/isdninfo
void Dialout::readisdninfo()
{
  isdninfophone = "";
  isdninfoidmap = "";

  QFile file(ISDNINFO);
  if (file.open(IO_ReadOnly))
    {
      QTextStream ts(&file);
      isdninfoidmap = ts.readLine();
      isdninfochmap = ts.readLine();
      isdninfodrmap = ts.readLine();
      isdninfousage = ts.readLine();
      isdninfoflags = ts.readLine();
      isdninfophone = ts.readLine();
      file.close();
    }
}

void Dialout::filldialog()
{
  QBoxLayout *toplayout = new QVBoxLayout(this, 8);

  QLabel *label = new QLabel(i18n("Active providers:"), this);
  toplayout->addWidget(label);

  QBoxLayout *list_button_layout = new QHBoxLayout(toplayout);

  // Listbox for current providers
  provider = new QListBox(this);
  connect(provider, SIGNAL(highlighted(const QString &)), this,
          SLOT(slotcheckconnect()));
  connect(provider, SIGNAL(doubleClicked(QListBoxItem *)), this,
          SLOT(slotcheckconnect()));
  connect(provider, SIGNAL(clicked(QListBoxItem *)), this,
          SLOT(slotcheckconnect()));
  label->setBuddy(provider);
  list_button_layout->addWidget(provider);

  // Buttons
  QBoxLayout *button_layout = new QVBoxLayout(list_button_layout, 3);
  b_connect = new QPushButton(i18n("&Connect"), this);
  QToolTip::add(b_connect, i18n("connect to selected provider"));
  connect(b_connect, SIGNAL(clicked()), this, SLOT(slotconnect()));
  b_connect->setEnabled(false);
  
  b_hangup = new QPushButton(i18n("&Hangup"), this);
  QToolTip::add(b_hangup, i18n("disconnect from your provider"));
  connect(b_hangup, SIGNAL(clicked()), this, SLOT(slothangup()));
  b_hangup->setEnabled(FALSE);

  b_isdnstart = new QPushButton(i18n("&Start"), this);
  QToolTip::add(b_isdnstart, i18n("start ISDN service"));
  connect(b_isdnstart, SIGNAL(clicked()), this, SLOT(slotisdnstart()));
  
  b_isdnstop = new QPushButton(i18n("S&top"), this);
  QToolTip::add(b_isdnstop, i18n("stop ISDN service"));
  connect(b_isdnstop, SIGNAL(clicked()), this, SLOT(slotisdnstop()));

  // adds buttons to layout
  button_layout->addWidget(b_connect);
  button_layout->addWidget(b_hangup);
  button_layout->addSpacing(20);
  button_layout->addStretch(1);
  button_layout->addWidget(b_isdnstart);
  button_layout->addWidget(b_isdnstop);

  // ISDN Status
  toplayout->addSpacing(8);
  QBoxLayout *isdnstatus_layout = new QHBoxLayout(toplayout);
  label = new QLabel(this);
  label->setText(i18n("ISDN state:"));
  label->setFont(QFont("Helvetica", 10));
  QToolTip::add(label, i18n("show the current ISDN line state"));
  
  isdnstatuslabel = new QLabel(this);
  isdnstatuslabel->setFrameStyle(QFrame::WinPanel | QFrame::Sunken);
//  isdnstatuslabel->setPaletteBackgroundColor(green);
//  isdnstatuslabel->setPaletteForegroundColor(black);
//  isdnstatuslabel->setText(i18n("  OFFLINE  "));
  isdnstatuslabel->setFont(QFont("Helvetica", 10));
  setisdnstatus(OFFLINE);
  isdnstatus_layout->addWidget(label);
  isdnstatus_layout->addWidget(isdnstatuslabel);
  isdnstatus_layout->addStretch(1);
  QToolTip::add(isdnstatuslabel, i18n("show the current ISDN line state"));

  // time connected status
  label = new QLabel(this);
  label->setText(i18n("Online since:"));
  label->setFont(QFont("Helvetica", 10));
  QToolTip::add(label, i18n("show the online time"));
  timeconnectedlabel = new QLabel(this);
  timeconnectedlabel->setText("  00:00:00  ");
  timeconnectedlabel->setFont(QFont("Helvetica", 10));
  timeconnectedlabel->setFrameStyle(QFrame::WinPanel | QFrame::Sunken);
  QToolTip::add(timeconnectedlabel, i18n("show the online time"));
  isdnstatus_layout->addWidget(label);
  isdnstatus_layout->addWidget(timeconnectedlabel);

  // button to reset online time counter
  b_counter = new QPushButton(this);
  b_counter->setPixmap(KGlobal::iconLoader()->loadIcon("reset"));
  connect(b_counter, SIGNAL(clicked()), this, SLOT(slotresetcounter()));
  QToolTip::add(b_counter, i18n("reset the online time"));
  isdnstatus_layout->addWidget(b_counter);
  
  for (ProviderData *pd = ISDNData.providerdata.first(); pd != 0;
       pd = ISDNData.providerdata.next())
    if (pd->isproviderenabled)
      provider->insertItem(KGlobal::iconLoader()->loadIcon("isp"),
                           pd->providername);
}

bool Dialout::is_trying()
{
  // state is dialing if flags contains 0
  if (isdninfoflags.contains("0"))
    return true;

  // online, start isdnstatustime only if the provider disconneted
  if (!is_isdnstatustimer) {
    isdnstatustimer->start(1000, false);
    is_isdnstatustimer = true;
  }
  return false;
}

void Dialout::errormessage()
{
  QString s = i18n("Could not connect to selected provider.\n");
  QMessageBox::warning(this, i18n("Connect to provider failed."), s);
}

Dialout::~Dialout()
{
}





