/* Copyright 2000 Red Hat, Inc.
 *
 * Than Ngo <than@redhat.de>
 *
 * This software may be freely redistributed under the terms of the GNU
 * public license.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


#include <unistd.h>
#include <stdlib.h>
#include <iostream>

#ifndef QTONLY
#  include <kglobal.h>
#  include <klocale.h>
#else
#  include "kde-int.h"
#endif

#include "isdn.h"

KIsdnConfig::KIsdnConfig(QWidget * parent,
			 const char *name):KCModule(parent, name)
{
  checkroot();
  ISDNData.load();

  QBoxLayout *toplayout = new QVBoxLayout(this, 5);
  tabdialog = new QTabWidget(this);
  toplayout->addWidget(tabdialog);

  QPixmap dial_pic = KGlobal::iconLoader()->loadIcon("dial");
  tab_dialout = new Dialout(this);
  tabdialog->addTab(tab_dialout, QIconSet(dial_pic), i18n("&Dial"));
  connect(tab_dialout, SIGNAL(setproviderisdncardtab(bool)),
          SLOT(slotsetproviderisdncardtab(bool)));

  QPixmap provider_pic = KGlobal::iconLoader()->loadIcon("city");
  tab_provider = new Provider(this);
  tabdialog->addTab(tab_provider, QIconSet(provider_pic), i18n("&Provider"));
  connect(tab_provider, SIGNAL(addproviderdialout(const QString &)),
          SLOT(slotaddproviderdialout(const QString &)));
  connect(tab_provider, SIGNAL(removeproviderdialout(const QString &)),
          SLOT(slotremoveproviderdialout(const QString &)));
  connect(tab_provider,
          SIGNAL(changeproviderdialout(const QString &, const QString &)),
          SLOT(slotchangeproviderdialout(const QString &, const QString &)));
  connect(tab_provider, SIGNAL(somethingChanged()), SLOT(slotconfigChanged()));

  QPixmap hardware_pic = KGlobal::iconLoader()->loadIcon("hardware");
  tab_isdncard = new ISDNCard(this);
  tabdialog->addTab(tab_isdncard, QIconSet(hardware_pic), i18n("&Hardware"));
  connect(tab_isdncard, SIGNAL(somethingChanged()), SLOT(slotconfigChanged()));

  QPixmap about_pic = KGlobal::iconLoader()->loadIcon("about");
  tab_about = new About(this);
  tabdialog->addTab(tab_about, QIconSet(about_pic), i18n("&About"));

  buttons();
}

KIsdnConfig::~KIsdnConfig()
{
}

void KIsdnConfig::slotconfigChanged()
{
  tabdialog->setTabEnabled(tab_dialout, false);	// disable dialout if somethingchanged
  emit changed(true);
}

void KIsdnConfig::checkroot()
{
  if (geteuid()) {
    QMessageBox::warning(this, "Red Hat ISDN config",
                         i18n("Only root is allowed to configure ISDN"),
                         QMessageBox::Ok, 0, 0);
    exit(1);
  }
  else setuid(0); // Be real root, in case some programs insist on it. -KK
}

void KIsdnConfig::load()
{
  emit changed(false);
}

void KIsdnConfig::save()
{
  tab_dialout->slotisdnstop();	// stop isdn service before save the changes
  ISDNData.save();		// save configuration
  if (!ISDNData.resourcedata.isenabled) {
    errormessage();
    // disable dialout after save changes, if no isdncard was configured
    tabdialog->setTabEnabled(tab_dialout, false);
  } else {
    if (tab_dialout->provider->count())
      // enable dialout after save changes and provider is configured
      tabdialog->setTabEnabled(tab_dialout, true);
    else
      // enable dialout after save changes and provider is configured
      tabdialog->setTabEnabled(tab_dialout, false);
  }
  emit changed(false);
}

void KIsdnConfig::defaults()
{
  emit changed(true);
}

int KIsdnConfig::buttons()
{
  setButtons(KCModule::Help | KCModule::Apply | KCModule::Ok | KCModule::Cancel);
  return KCModule::Help | KCModule::Apply | KCModule::Ok | KCModule::Cancel;
}

void KIsdnConfig::slotchangeproviderdialout(const QString & _old,
					    const QString & _new)
{
  tab_dialout->changeprovidername(_old, _new);
  slotconfigChanged();
}

void KIsdnConfig::slotremoveproviderdialout(const QString & s)
{
  tab_dialout->removeprovidername(s);
  slotconfigChanged();
}

void KIsdnConfig::slotsetproviderisdncardtab(bool b)
{
  tabdialog->setTabEnabled(tab_provider, b);
  tabdialog->setTabEnabled(tab_isdncard, b);
}

void KIsdnConfig::slotaddproviderdialout(const QString & s)
{
  tab_dialout->addprovidername(s);
  slotconfigChanged();
}

void KIsdnConfig::errormessage()
{
  QString s = i18n("The ISDN card is not configured.\n"
                   "Please choose your ISDN card from the\n"
                   "list and configure it.\n\n"
                   "Choose 'Cancel' if you don't want to\n"
                   "configure ISDN dialup connections");
  
  QMessageBox::warning(this, i18n("ISDN card was not configured"), s);
}

extern "C" { KCModule *create_isdn(QWidget * parent, const char *name)
{
  //KGlobal::locale()->insertCatalogue("isdn-config");
  return new KIsdnConfig(parent, name);
};

}
