/* Copyright 2000 Red Hat, Inc.
 *
 * Than Ngo <than@redhat.de>
 *
 * This software may be freely redistributed under the terms of the GNU
 * public license.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef QTONLY
#  include <kglobal.h>
#  include <klocal.h>
#else
#  include "kde-int.h"
#endif

#include <stdlib.h>
#include <iostream.h>

#include "isdndb.h"

enum { X75I, X75UI, X75BUI, HDLC };

QList <ISDNDBData> isdndbdata;

ISDNDBData::ISDNDBData()
{
  setNull();
}

ISDNDBData::~ISDNDBData()
{
} 

void ISDNDBData::setNull()
{
  country = "";
  city = "";
  flag = "";
  name = "";
  user = "";
  pass = "";
  prefix = "";
  phone = "";
  domain = "";
  dns = "";
  netmask = "";
  encap = 0;
  layer = 0;
  authtype = 0;
  ipsetup = true;
}

ISDNDB::ISDNDB(QString filename)
{
  isdndbdata.clear();
  ReadIsdnDatabase(filename);
}


void ISDNDB::ReadIsdnDatabase(QString filename)
{
  QFile file(filename);
  if (!file.open(IO_ReadOnly)) {
    cerr << filename << " is not available!" << endl;
    return;
  }

  QTextStream ts(&file);
  QString s;

  while (!ts.eof()) {
    s = ts.readLine();
    s = s.simplifyWhiteSpace();
    
    if (s.left(1) == "#" || s.isEmpty())
      continue;

    if (s.left(7) != "[Begin]") {
      cerr << "error: expect [Begin]" << endl;
      exit(1);
    }
    else {
      ISDNDBData *isdndb = new ISDNDBData();
      while (true) {
        if (s.left(9) == "[Country]")
          isdndb->country = getvalue(s);
        else if (s.left(6) == "[Flag]")
          isdndb->flag = getvalue(s);
        else if (s.left(6) == "[City]")
          isdndb->city = getvalue(s);
        else if (s.left(6) == "[Name]")
          isdndb->name = getvalue(s);
        else if (s.left(6) == "[User]")
          isdndb->user = getvalue(s);
        else if (s.left(6) == "[Pass]")
          isdndb->pass = getvalue(s);
        else if (s.left(8) == "[Prefix]")
          isdndb->prefix = getvalue(s);
        else if (s.left(7) == "[Phone]")
          isdndb->phone = getvalue(s);
        else if (s.left(8) == "[Domain]")
          isdndb->domain = getvalue(s);
        else if (s.left(5) == "[DNS]")
          isdndb->dns = getvalue(s);
        else if (s.left(9) == "[Netmask]")
          isdndb->netmask = getvalue(s);
        else if (s.left(8) == "[Encaps]") {
          if (getvalue(s).lower() == "syncppp")
            isdndb->encap = GeneralData::SYNCPPP;
          else
            isdndb->encap = GeneralData::RAWIP;
        }
        else if (s.left(8) == "[Layer2]")
          isdndb->layer = HDLC;
        else if (s.left(6) == "[Auth]") {
          if (getvalue(s).lower() == "pap")
            isdndb->authtype = GeneralData::AUTH_PAP;
          else if (getvalue(s).lower() == "chap")
            isdndb->authtype = GeneralData::AUTH_CHAP;
          else if (getvalue(s).lower() == "both")
            isdndb->authtype = GeneralData::AUTH_BOTH;
          else if (getvalue(s).lower() == "none")
            isdndb->authtype = GeneralData::AUTH_NONE;
        }
        else if (s.left(9) == "[Ipsetup]") {
          if (getvalue(s).lower() == "dynamic")
            isdndb->ipsetup = true;
          else
            isdndb->ipsetup = false;
        }
        else if (s.left(5) == "[End]")
          break;
        s = ts.readLine();
        s = s.simplifyWhiteSpace();
      }
      isdndbdata.append(isdndb); }
  }
  file.close();
  return;
}

QString ISDNDB::getvalue(QString str)
{
  int pos = str.find(' ');
  if (pos < 0)
    return(QString(""));

  return(str.remove(0, pos).stripWhiteSpace());
}

ISDNDB::~ISDNDB()
{
}
