/* Copyright 2000 Red Hat, Inc.
 *
 * Than Ngo <than@redhat.de>
 *
 * This software may be freely redistributed under the terms of the GNU
 * public license.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


#ifndef QTONLY
#  include <kiconloader.h>
#  include <klocale.h>
#else
#  include "kde-int.h"
#endif

#include <iostream>
#include <qpixmap.h>

#include "provider.h"

Provider::Provider(QWidget * parent, const char *name):QWidget(parent,name)
{
  QBoxLayout *toplayout = new QVBoxLayout(this, 8);

  QLabel *label = new QLabel(i18n("Current providers:"), this);
  toplayout->addWidget(label);

  QBoxLayout *list_button_layout = new QHBoxLayout(toplayout);

  // Listbox for current provider
  provider = new QListBox(this);
  label->setBuddy(provider);
  list_button_layout->addWidget(provider);
  connect(provider, SIGNAL(highlighted(const QString &)), this,
          SLOT(slotprovider()));
  connect(provider, SIGNAL(doubleClicked(QListBoxItem *)), this,
          SLOT(slotEdit(QListBoxItem *)));
  connect(provider, SIGNAL(returnPressed(QListBoxItem *)), this,
          SLOT(slotEdit(QListBoxItem *)));

  // Buttons
  QBoxLayout *button_layout = new QVBoxLayout(list_button_layout, 3);
  new_button = new QPushButton(i18n("Create &new"), this);
  QToolTip::add(new_button, i18n("Create a new provider"));
  connect(new_button, SIGNAL(clicked()), this, SLOT(slotNew()));

  add_button = new QPushButton(i18n("Ch&oose"), this);
  QToolTip::add(add_button,
                i18n("Choose a provider from isdn database"));
  connect(add_button, SIGNAL(clicked()), this, SLOT(slotAdd()));

  delete_button = new QPushButton(i18n("&Delete"), this);
  QToolTip::add(delete_button,
                i18n("Delete the selected provider from the list"));
  connect(delete_button, SIGNAL(clicked()), this, SLOT(slotDelete()));
  delete_button->setEnabled(FALSE);

  edit_button = new QPushButton(i18n("&Edit"), this);
  QToolTip::add(edit_button,
                i18n("Edit the selected provider from the list"));
  connect(edit_button, SIGNAL(clicked()), this, SLOT(slotEdit()));
  edit_button->setEnabled(FALSE);

  button_layout->addWidget(new_button);
  button_layout->addWidget(add_button);
  button_layout->addWidget(edit_button);
  button_layout->addStretch(1);
  button_layout->addWidget(delete_button);
  
  load();
}

void Provider::load()
{
  QString pic;
  for (ProviderData *pd = ISDNData.providerdata.first(); pd != 0;
       pd = ISDNData.providerdata.next()) {
    pic = pd->isproviderenabled ? "isp" : "isp-disabled";
    provider->insertItem(KGlobal::iconLoader()->loadIcon(pic), pd->providername);
  }
  checkbuttons();
}

void Provider::setbuttons(bool t)
{
  delete_button->setEnabled(t);
  edit_button->setEnabled(t);
}

bool Provider::isproviderselected()
{
  if (provider->currentItem() < 0)
    return FALSE;
  else
    return TRUE;
}

void Provider::addprovider(const QString &s)
{
  provider->insertItem(KGlobal::iconLoader()->loadIcon("isp"), s);
  checkbuttons();
  emit somethingChanged();
}

void Provider::addprovider(const QString icon, const QString &s)
{
  provider->insertItem(KGlobal::iconLoader()->loadIcon(icon), s);
  checkbuttons();
  emit somethingChanged();
}

void Provider::editprovider(const QString &s)
{
  providerinfo = new ProviderInfo();
  providerinfo->fillproviderinfo(s);

  if (providerinfo->exec() == QDialog::Accepted) {
    // if provider enabled, add provider in dial tabwidget
    if (providerinfo->activeprovider->isChecked()) {
      if (providerinfo->oldisproviderenabled) {
        provider->changeItem(KGlobal::iconLoader()->
                             loadIcon("isp"),
                             providerinfo->providername->text(),
                             provider->currentItem());
        emit changeproviderdialout(s,
                                   providerinfo->providername->
                                   text());
      }
      else {
        provider->changeItem(KGlobal::iconLoader()->
                             loadIcon("isp"),
                             providerinfo->providername->text(),
                             provider->currentItem());
        emit addproviderdialout(providerinfo->providername->
                                text());
      }
    }
    else {
      if (providerinfo->oldisproviderenabled) {
        provider->changeItem(KGlobal::iconLoader()->
                             loadIcon("isp-disabled"),
                             providerinfo->providername->text(),
                             provider->currentItem());
        emit removeproviderdialout(s);
      }
      else
        provider->changeItem(KGlobal::iconLoader()->
                             loadIcon("isp-disabled"),
                             providerinfo->providername->text(),
                             provider->currentItem());
  
      emit somethingChanged();
    }
  }
  
  delete providerinfo;
  checkbuttons();
}

void Provider::checkbuttons()
{
  setbuttons(isproviderselected());
}

void Provider::slotprovider()
{
  checkbuttons();
}

void Provider::slotNew()
{
  providerinfo = new ProviderInfo();
  if (providerinfo->exec() == QDialog::Accepted) {
    // if provider enabled, add provider in dial tabwidget
    if (providerinfo->activeprovider->isChecked()) {
      addprovider(providerinfo->providername->text());
      emit addproviderdialout(providerinfo->providername->text());
    }
    else
      addprovider("isp-disabled", providerinfo->providername->text());
  }
  delete providerinfo;
}

void Provider::slotAdd()
{
  QFile file(ISDNDATABASE);

  if (!file.exists()) {
    noisdndbmessage();
    return;
  }

  providerdb = new ProviderDb(ISDNDATABASE);

  if (providerdb->exec() == QDialog::Accepted) {
    ProviderData *t = new ProviderData();

    t->loginname = providerdb->selectEntry()->user;
    t->password = providerdb->selectEntry()->pass;

    // add the provider name to the list
    addprovider(providerdb->selectEntry()->name);

    // Provider
    t->providername = providerdb->selectEntry()->name;
    // t->city = "";
    // t->country = "";
    t->areacode = providerdb->selectEntry()->prefix;
    t->phonenumber = providerdb->selectEntry()->phone;
    // t->huptimeout = 120;
    t->msn = providerdb->msn;
    
    if (providerdb->selectEntry()->encap == GeneralData::SYNCPPP) {
      t->encapsulation = "syncppp";
      t->isproviderenabled = true;
      t->dialmode = "manual";
    }
    else {
      t->encapsulation = "rawip";
      t->isproviderenabled = false;
      t->dialmode = "auto";
    }

    switch (providerdb->selectEntry()->authtype) {
    case GeneralData::AUTH_PAP:
      t->auth = "+pap -chap";
      break;
    case GeneralData::AUTH_CHAP:
      t->auth = "-pap +chap";
      break;
    case GeneralData::AUTH_BOTH:
      t->auth = "+pap +chap";
      break;
    case GeneralData::AUTH_NONE:
      t->auth = "-pap -chap";
    }
    
    // Network
    t->dns = providerdb->selectEntry()->dns;
    if (providerdb->selectEntry()->domain)
      t->domain = providerdb->selectEntry()->domain;
    else
      t->domain = "";
    
    if (providerdb->selectEntry()->ipsetup) {
      t->islocalipenabled = FALSE;
      t->isremoteipenabled = FALSE;
    }
    else {
      t->islocalipenabled = TRUE;
      t->isremoteipenabled = TRUE;
      t->localip = "";
      t->remoteip = "";
    }
    
    // adds new provider in Qlist
    ISDNData.providerdata.append(t);
    emit addproviderdialout(providerdb->selectEntry()->name);
  }

  delete providerdb;
}

void Provider::slotDelete()
{
  QString s = i18n("Do you really want to delete the\n"
                   "selected Provider ?\n\n");

  if (QMessageBox::
      warning(this, i18n("Delete provider"), s, i18n("Yes"), i18n("No"),
							0, 1) == 0) {
    QString providername;

    // take current provider from Qlist
    for (ProviderData *t = ISDNData.providerdata.first(); t != 0;
         t = ISDNData.providerdata.next()) {
      if (t->providername == provider->currentText())
        break;
    }
		
    ISDNData.providerdata.remove();
    providername = provider->currentText();
    provider->removeItem(provider->currentItem());
    checkbuttons();
    emit removeproviderdialout(providername);
  }
}

void Provider::slotEdit(QListBoxItem * item)
{
  editprovider(item->text());
}

void Provider::slotEdit()
{
  editprovider(provider->text(provider->currentItem()));
}

void Provider::noisdndbmessage()
{
  QString s = i18n("The ISDN database ");
  s += QString(ISDNDATABASE);
  s += "\n";
  s += i18n("is not available!\n");

  QMessageBox::warning(this, i18n("no ISDN database"), s);
}

Provider::~Provider()
{
}
