/* Copyright 2000 Red Hat, Inc.
 *
 * Than Ngo <than@redhat.de>
 *
 * This software may be freely redistributed under the terms of the GNU
 * public license.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


#ifndef QTONLY
#  include <klocale.h>
#  include <kbuttonbox.h>
#  include <kiconloader.h>
#  include <kglobal.h>
#  include <kstddirs.h>
#else
#  include "kde-int.h"
#endif

#include <qlayout.h>
#include <qpixmap.h>
#include <qfile.h>
#include <qmessagebox.h>

#include <iostream>
#include <stdio.h>

#include "providerdb.h"
#include "generaldata.h"
#include "passworddialog.h"

ProviderDb::ProviderDb(const char *isdndb, QWidget * parent,
		       const char *name):QDialog(parent, name, TRUE)
{
  static QPixmap flag_xpm;
  static QPixmap logo_xpm;
  static QPixmap isp_xpm = KGlobal::iconLoader()->loadIcon("isp");
  static QPixmap city_xpm = KGlobal::iconLoader()->loadIcon("city");

  dbfile = QString(isdndb);
  setFocusPolicy(StrongFocus);
  setCaption(i18n("ISDN provider database"));

  QBoxLayout *toplayout = new QVBoxLayout(this, 8);

  TreeList = new QListView(this);
  TreeList->setMinimumSize(300, 300);
  TreeList->setSorting(1, false);

  toplayout->addWidget(TreeList);
  TreeList->addColumn(i18n("Provider name"));
  TreeList->setRootIsDecorated(TRUE);
  
  // open isdn database
  db = new ISDNDB(dbfile);

  // add Items in Listview Widget
  QString _country = "";
  QString _city = "";
  for (ISDNDBData *entry=isdndbdata.first(); entry != 0; entry=isdndbdata.next()) {
    if (_country != entry->country) {
      if (!entry->flag.isEmpty())
        flag_xpm = KGlobal::iconLoader()->loadIcon(entry->flag);
      else
        flag_xpm = isp_xpm;
    
      country = new QListViewItem(TreeList, entry->country);
      country->setPixmap(0, flag_xpm);
      _country = entry->country;
    }

    if (_city != entry->city) {
      city = new QListViewItem(country, entry->city);
      city->setPixmap(0, city_xpm);
      _city = entry->city;
    }

    pname = new QListViewItem(city, entry->name);
    pname->setPixmap(0, isp_xpm);
  }

  // Horizontal Line
  QFrame *tmpQFrame = new QFrame(this);
  tmpQFrame->setFrameStyle(QFrame::HLine | QFrame::Sunken);
  toplayout->addWidget(tmpQFrame);

  // Button Ok, Cancel
  KButtonBox *bbox = new KButtonBox(this);
  bbox->addStretch(10);

  OK = bbox->addButton(i18n("&OK"));
  connect(OK, SIGNAL(clicked()), SLOT(accept()));

  Cancel = bbox->addButton(i18n("&Cancel"));
  connect(Cancel, SIGNAL(clicked()), SLOT(reject()));

  bbox->layout();
  toplayout->addWidget(bbox);
}

void ProviderDb::noProviderSelected()
{
  QString s = i18n("You should select a internet provider\n"
                   "in the list and then press 'OK'.\n\n"
                   "Choose 'Cancel' if you don't want to\n"
                   "create a new account.");

  QMessageBox::warning(this, i18n("Select a provider"), s);
}

void ProviderDb::accept()
{
  ISDNDBData *entry;
  
  PasswordDialog passworddialog;

  provideritem = TreeList->currentItem();
  // no provider selected
  if (provideritem == 0) {
    noProviderSelected();
    return;
  }

  // Provideritem is empty
  if (provideritem->text(0).length() == 0) {
    noProviderSelected();
    return;
  }

  // Provider selected
  if (provideritem->childCount() == 0) {
    QString _country = provideritem->parent()->parent()->text(0);
    QString _city = provideritem->parent()->text(0);
    QString _providername = provideritem->text(0);

    for (entry=isdndbdata.first(); entry != 0; entry=isdndbdata.next()) {
      if (entry->country == _country && entry->city == _city && entry->name == _providername) {
        select = entry;
      }
    }
  }
  else {
    noProviderSelected();
    return;
  }
  
  if (ISDNData.isproviderexisted(selectEntry()->name)) {
    QString s = i18n("The provider name exists already.\n"
                     "Please choose another provider.\n\n");
    QMessageBox::warning(this, i18n("Provider exists"), s);
    return;
  }
  
  passworddialog.fill(selectEntry()->user, selectEntry()->pass);
  if (passworddialog.exec() == QDialog::Accepted) {
    msn = passworddialog.getmsn();
    selectEntry()->user = QString(passworddialog.getloginname());
    selectEntry()->pass = QString(passworddialog.getpassword());
    done(Accepted);
  }
  else
    return;
}

ProviderDb::~ProviderDb()
{
}
